@@ -23,6 +23,10 @@
 #include <QFontDialog>
 #include <QStyleFactory>
 #include <QMessageBox>
+#if defined _WIN32 && (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+#	include <QApplication>
+#	include <QStyleHints>
+#endif
 #include <QDebug>
 
 #include "qalculateqtsettings.h"
@@ -121,7 +125,11 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	combo->setCurrentIndex(settings->style < 0 ? 0 : settings->style + 1);
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(styleChanged(int)));
 	l2->addWidget(combo, r, 1); r++; styleCombo = combo;
+#if defined _WIN32 && (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+	BOX_G(tr("Dark mode"), settings->palette == 1 || (settings->palette == -1 && QGuiApplication::styleHints()->colorScheme() == Qt::ColorScheme::Dark), darkModeToggled(bool));
+#else
 	BOX_G(tr("Dark mode"), settings->palette == 1, darkModeToggled(bool));
+#endif
 	BOX_G(tr("Colorize result"), settings->colorize_result, colorizeToggled(bool));
 	BOX_G(tr("Format result"), settings->format_result, formatToggled(bool));
 	BOX_G1(tr("Custom result font:"), settings->use_custom_result_font, resultFontToggled(bool)); 
@@ -170,6 +178,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	combo->setCurrentIndex(combo->findData(settings->replace_expression));
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(replaceExpressionChanged(int)));
 	l2->addWidget(combo, r, 1); r++;
+	BOX_G(tr("Automatically copy result"), settings->autocopy_result, autocopyResultToggled(bool));
 	l2->addWidget(new QLabel(tr("Parsing mode:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("Adaptive"), PARSING_MODE_ADAPTIVE);
@@ -284,6 +293,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	BOX_G(tr("Enable all SI-prefixes"), settings->printops.use_all_prefixes, allPrefixesToggled(bool));
 	BOX_G(tr("Enable denominator prefixes"), settings->printops.use_denominator_prefix, denominatorPrefixToggled(bool));
 	BOX_G(tr("Enable units in physical constants"), CALCULATOR->variableUnitsEnabled(), variableUnitsToggled(bool));
+	BOX_G(tr("Copy unformatted ASCII without units"), settings->copy_ascii_without_units, copyAsciiWithoutUnitsToggled(bool));
 	l2->addWidget(new QLabel(tr("Temperature calculation:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("Absolute"), TEMPERATURE_CALCULATION_ABSOLUTE);
@@ -448,7 +458,7 @@ void PreferencesDialog::ignoreCommaToggled(bool b) {
 }
 void PreferencesDialog::colorizeToggled(bool b) {
 	settings->colorize_result = b;
-	emit resultDisplayUpdated();
+	emit historyExpressionTypeChanged();
 }
 void PreferencesDialog::formatToggled(bool b) {
 	settings->format_result = b;
@@ -521,6 +531,9 @@ void PreferencesDialog::repeatingDecimalsToggled(bool b) {
 void PreferencesDialog::copyAsciiToggled(bool b) {
 	settings->copy_ascii = b;
 }
+void PreferencesDialog::copyAsciiWithoutUnitsToggled(bool b) {
+	settings->copy_ascii_without_units = b;
+}
 void PreferencesDialog::caretAsXorToggled(bool b) {
 	settings->caret_as_xor = b;
 }
@@ -648,25 +661,24 @@ void PreferencesDialog::darkModeToggled(bool b) {
 	else settings->palette = -1;
 #ifdef _WIN32
 	if(b) {
-		if(styleCombo->currentIndex() == 0 || styleCombo->currentText().compare("windowsvista", Qt::CaseInsensitive) == 0) {
+		if(styleCombo->currentText().compare("windowsvista", Qt::CaseInsensitive) == 0) {
 			for(int i = 1; i < styleCombo->count(); i++) {
 				if(styleCombo->itemText(i).compare("Fusion", Qt::CaseInsensitive) == 0) {
-					int prev_style = settings->style;
 					styleCombo->setCurrentIndex(i);
-					settings->light_style = prev_style;
 					break;
 				}
 			}
 		}
-	} else if(settings->light_style != settings->style && styleCombo->currentText().compare("Fusion", Qt::CaseInsensitive) == 0) {
-		styleCombo->setCurrentIndex(settings->light_style < 0 ? 0 : settings->light_style + 1);
 	}
 #endif
+#if defined _WIN32 && (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+	settings->updatePalette(QGuiApplication::styleHints()->colorScheme() != Qt::ColorScheme::Dark);
+#else
 	settings->updatePalette();
+#endif
 }
 void PreferencesDialog::styleChanged(int i) {
 	settings->style = i - 1;
-	settings->light_style = settings->style;
 	settings->updateStyle();
 }
 void PreferencesDialog::factorizeToggled(bool b) {
@@ -681,6 +693,9 @@ void PreferencesDialog::closeEvent(QCloseEvent *e) {
 void PreferencesDialog::replaceExpressionChanged(int i) {
 	settings->replace_expression = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
 }
+void PreferencesDialog::autocopyResultToggled(bool b) {
+	settings->autocopy_result = b;
+}
 void PreferencesDialog::historyExpressionChanged(int i) {
 	settings->history_expression_type = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
 	emit historyExpressionTypeChanged();
