@@ -21,6 +21,9 @@
 #include <QColor>
 #include <QProgressDialog>
 #include <QKeySequence>
+#if defined _WIN32 && (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+#	include <QStyleHints>
+#endif
 #include <QDebug>
 
 #define PREFERENCES_VERSION_BEFORE(i1, i2, i3) (preferences_version[0] < i1 || (preferences_version[0] == i1 && (preferences_version[1] < i2 || (preferences_version[1] == i2 && preferences_version[2] < i3))))
@@ -194,13 +197,29 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 		int ks_type = 0;
 		int n = sscanf(svalue.c_str(), "%s %i %[^\n]", str1, &ks_type, str2);
 		if(n >= 2 && ks_type >= SHORTCUT_TYPE_FUNCTION && ks_type <= LAST_SHORTCUT_TYPE) {
-			keyboard_shortcut *ks = new keyboard_shortcut;
-			if(n == 3) {ks->value = str2; remove_blank_ends(ks->value);}
-			ks->type = (shortcut_type) ks_type;
-			ks->key = str1;
-			ks->new_action = false;
-			ks->action = NULL;
-			keyboard_shortcuts.push_back(ks);
+			keyboard_shortcut *ks = NULL;
+			for(size_t i = 0; i < keyboard_shortcuts.size(); i++) {
+				if(keyboard_shortcuts[i]->key == str1) {
+					ks = keyboard_shortcuts[i];
+					break;
+				}
+			}
+			if(!ks || !PREFERENCES_VERSION_BEFORE(4, 7, 0)) {
+				if(!ks) {
+					ks = new keyboard_shortcut;
+					ks->key = str1;
+					ks->new_action = false;
+					ks->action = NULL;
+					keyboard_shortcuts.push_back(ks);
+				}
+				if(n == 3) {
+					ks->value.push_back(str2);
+					if(ks_type != SHORTCUT_TYPE_TEXT) remove_blank_ends(ks->value[ks->value.size() - 1]);
+				} else {
+					ks->value.push_back("");
+				}
+				ks->type.push_back((shortcut_type) ks_type);
+			}
 		}
 	} else if(!is_workspace && svar == "custom_button_label") {
 		int c = 0, r = 0;
@@ -245,8 +264,12 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 			}
 			index--;
 			custom_buttons[index].type[bi] = cb_type;
-			if(n == 5) {custom_buttons[index].value[bi] = str; remove_blank_ends(custom_buttons[index].value[bi]);}
-			else custom_buttons[index].value[bi] = "";
+			if(n == 5) {
+				custom_buttons[index].value[bi] = str;
+				if(cb_type != SHORTCUT_TYPE_TEXT) remove_blank_ends(custom_buttons[index].value[bi]);
+			} else {
+				custom_buttons[index].value[bi] = "";
+			}
 		}
 	} else if(svar == "keypad_type") {
 		if(v >= 0 && v <= 3) keypad_type = v;
@@ -268,6 +291,8 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 		printops.use_max_decimals = v;
 	} else if(svar == "precision") {
 		CALCULATOR->setPrecision(v);
+	} else if(svar == "previous_precision") {
+		previous_precision = v;
 	} else if(svar == "min_exp") {
 		printops.min_exp = v;
 	} else if(svar == "interval_arithmetic") {
@@ -349,9 +374,11 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 			printops.allow_factorization = (evalops.structuring == STRUCTURING_FACTORIZE);
 		}
 	} else if(svar == "angle_unit") {
-		if(v >= ANGLE_UNIT_NONE && v <= ANGLE_UNIT_GRADIANS) {
+		if(v >= ANGLE_UNIT_NONE && v <= ANGLE_UNIT_CUSTOM) {
 			evalops.parse_options.angle_unit = (AngleUnit) v;
 		}
+	} else if(svar == "custom_angle_unit") {
+		custom_angle_unit = svalue;
 	} else if(svar == "functions_enabled") {
 		evalops.parse_options.functions_enabled = v;
 	} else if(svar == "variables_enabled") {
@@ -478,6 +505,8 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 			window_state = QByteArray::fromBase64(svalue.c_str());
 		} else if(svar == "replace_expression") {
 			replace_expression = v;
+		} else if(svar == "autocopy_result") {
+			autocopy_result = v;
 		} else if(svar == "history_expression_type") {
 			history_expression_type = v;
 		} else if(svar == "window_geometry") {
@@ -509,11 +538,21 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 		} else if(svar == "datasets_hsplitter_state") {
 			datasets_hsplitter_state = QByteArray::fromBase64(svalue.c_str());
 		} else if(svar == "style") {
+#if defined _WIN32
+			if(!PREFERENCES_VERSION_BEFORE(4, 7, 0)) style = v;
+#else
 			style = v;
-		} else if(svar == "light_style") {
-			light_style = v;
+#endif
 		} else if(svar == "palette") {
+#ifdef _WIN32
+#if	(QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+			if(!PREFERENCES_VERSION_BEFORE(4, 7, 0) || (v == 1 && QGuiApplication::styleHints()->colorScheme() != Qt::ColorScheme::Dark)) palette = v;
+#	else
+			if(!PREFERENCES_VERSION_BEFORE(4, 7, 0)) palette = v;
+#	endif
+#else
 			palette = v;
+#endif
 		} else if(svar == "color") {
 			colorize_result = v;
 		} else if(svar == "format") {
@@ -643,6 +682,8 @@ void QalculateQtSettings::readPreferenceValue(const std::string &svar, const std
 			caret_as_xor = v;
 		} else if(svar == "copy_ascii") {
 			copy_ascii = v;
+		} else if(svar == "copy_ascii_without_units") {
+			copy_ascii_without_units = v;
 		} else if(svar == "decimal_comma") {
 			decimal_comma = v;
 			if(v == 0) CALCULATOR->useDecimalPoint(evalops.parse_options.comma_as_separator);
@@ -683,6 +724,7 @@ void QalculateQtSettings::loadPreferences() {
 	f_answer = CALCULATOR->addFunction(new AnswerFunction());
 
 	CALCULATOR->setPrecision(10);
+	previous_precision = 0;
 	CALCULATOR->useIntervalArithmetic(true);
 	CALCULATOR->setTemperatureCalculationMode(TEMPERATURE_CALCULATION_HYBRID);
 	CALCULATOR->useBinaryPrefixes(0);
@@ -749,6 +791,7 @@ void QalculateQtSettings::loadPreferences() {
 	evalops.parse_options.limit_implicit_multiplication = false;
 	evalops.parse_options.parsing_mode = PARSING_MODE_ADAPTIVE;
 	evalops.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
+	custom_angle_unit = "";
 	evalops.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
 	evalops.parse_options.comma_as_separator = false;
 	evalops.mixed_units_conversion = MIXED_UNITS_CONVERSION_DEFAULT;
@@ -773,6 +816,7 @@ void QalculateQtSettings::loadPreferences() {
 	rpn_shown = false;
 	caret_as_xor = false;
 	copy_ascii = false;
+	copy_ascii_without_units = false;
 	do_imaginary_j = false;
 	simplified_percentage = true;
 	color = 1;
@@ -806,9 +850,9 @@ void QalculateQtSettings::loadPreferences() {
 	custom_keypad_font = "";
 	custom_app_font = "";
 	style = -1;
-	light_style = -1;
 	palette = -1;
 	replace_expression = KEEP_EXPRESSION;
+	autocopy_result = false;
 	save_mode_on_exit = true;
 	save_defs_on_exit = true;
 	clear_history_on_exit = false;
@@ -871,8 +915,8 @@ void QalculateQtSettings::loadPreferences() {
 	max_plot_time = 5;
 
 	preferences_version[0] = 4;
-	preferences_version[1] = 6;
-	preferences_version[2] = 1;
+	preferences_version[1] = 7;
+	preferences_version[2] = 0;
 
 	if(file) {
 		char line[1000000L];
@@ -902,7 +946,7 @@ void QalculateQtSettings::loadPreferences() {
 	}
 
 	keyboard_shortcut *ks;
-#define ADD_SHORTCUT(k, t, v) ks = new keyboard_shortcut; ks->key = k; ks->type = t; ks->value = v; ks->action = NULL; ks->new_action = false; keyboard_shortcuts.push_back(ks);
+#define ADD_SHORTCUT(k, t, v) ks = new keyboard_shortcut; ks->key = k; ks->type.push_back(t); ks->value.push_back(v); ks->action = NULL; ks->new_action = false; keyboard_shortcuts.push_back(ks);
 	if(default_shortcuts) {
 #ifdef _WIN32
 		ADD_SHORTCUT("Ctrl+Q", SHORTCUT_TYPE_QUIT, "")
@@ -960,15 +1004,25 @@ void QalculateQtSettings::loadPreferences() {
 	v_memory->addName(ename);
 	CALCULATOR->addVariable(v_memory);
 
-	if(palette != 1) light_style = style;
+#ifdef _WIN32
+	updateStyle();
+#else
 	if(style >= 0) updateStyle();
 	else if(palette >= 0) updatePalette();
+#endif
 
 	if(!current_workspace.empty()) {
 		if(!loadWorkspace(current_workspace.c_str())) current_workspace = "";
 	}
 
 }
+void QalculateQtSettings::setCustomAngleUnit() {
+	if(!custom_angle_unit.empty()) {
+		CALCULATOR->setCustomAngleUnit(CALCULATOR->getActiveUnit(custom_angle_unit));
+		if(CALCULATOR->customAngleUnit()) custom_angle_unit = CALCULATOR->customAngleUnit()->referenceName();
+	}
+	if(evalops.parse_options.angle_unit == ANGLE_UNIT_CUSTOM && !CALCULATOR->customAngleUnit()) evalops.parse_options.angle_unit = ANGLE_UNIT_NONE;
+}
 void QalculateQtSettings::updateFavourites() {
 	if(favourite_functions_pre.empty()) {
 		for(size_t i = 0; i < CALCULATOR->functions.size(); i++) {
@@ -1013,9 +1067,17 @@ void QalculateQtSettings::updateFavourites() {
 		}
 	}
 }
-void QalculateQtSettings::updatePalette() {
+
+#if defined _WIN32 && (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+void QalculateQtSettings::updatePalette(bool force_update) {
+	if(!force_update && palette != 1 && QGuiApplication::styleHints()->colorScheme() != Qt::ColorScheme::Dark && (style < 0 || style >= QStyleFactory::keys().count() || QStyleFactory::keys().at(style).compare("Fusion", Qt::CaseInsensitive) == 0)) return;
+	QPalette p;
+	if(palette == 1 && QGuiApplication::styleHints()->colorScheme() != Qt::ColorScheme::Dark) {
+#else
+void QalculateQtSettings::updatePalette(bool) {
 	QPalette p;
 	if(palette == 1) {
+#endif
 		p.setColor(QPalette::Active, QPalette::Window, QColor(42, 46, 50));
 		p.setColor(QPalette::Active, QPalette::WindowText, QColor(252, 252, 252));
 		p.setColor(QPalette::Active, QPalette::Base, QColor(27, 30, 32));
@@ -1056,7 +1118,18 @@ void QalculateQtSettings::updatePalette() {
 		p.setColor(QPalette::Disabled, QPalette::ButtonText,QColor(101, 101, 101));
 		p.setColor(QPalette::Disabled, QPalette::BrightText, QColor(39, 174, 96));
 	} else {
+#ifdef _WIN32
+		QStyle *s = NULL;
+#if	(QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+		if(QGuiApplication::styleHints()->colorScheme() != Qt::ColorScheme::Dark || style < 0 || style >= QStyleFactory::keys().count() || QStyleFactory::keys().at(style).compare("windowsvista", Qt::CaseInsensitive) != 0) s = QStyleFactory::create("fusion");
+#else
+		s = QStyleFactory::create("fusion");
+#endif
+		if(!s) s = QApplication::style();
+		p = s->standardPalette();
+#else
 		p = QApplication::style()->standardPalette();
+#endif
 	}
 	QApplication::setPalette(p);
 }
@@ -1067,7 +1140,7 @@ void QalculateQtSettings::updateStyle() {
 	}
 #ifdef _WIN32
 	else {
-		QStyle *s = QStyleFactory::create("windowsvista");
+		QStyle *s = QStyleFactory::create("fusion");
 		if(s) QApplication::setStyle(s);
 	}
 #endif
@@ -1168,9 +1241,6 @@ bool QalculateQtSettings::savePreferences(const char *filename, bool is_workspac
 		fprintf(file, "completion_min2=%i\n", completion_min2);
 		fprintf(file, "completion_delay=%i\n", completion_delay);
 		fprintf(file, "style=%i\n", style);
-#ifdef _WIN32
-		if(light_style != style && palette == 1) fprintf(file, "light_style=%i\n", light_style);
-#endif
 		fprintf(file, "palette=%i\n", palette);
 		fprintf(file, "color=%i\n", colorize_result);
 		if(!format_result) fprintf(file, "format=%i\n", format_result);
@@ -1186,6 +1256,7 @@ bool QalculateQtSettings::savePreferences(const char *filename, bool is_workspac
 		if(printops.division_sign != DIVISION_SIGN_DIVISION_SLASH) fprintf(file, "division_sign=%i\n", printops.division_sign);
 		if(implicit_question_asked) fprintf(file, "implicit_question_asked=%i\n", implicit_question_asked);
 		fprintf(file, "replace_expression=%i\n", replace_expression);
+		fprintf(file, "autocopy_result=%i\n", autocopy_result);
 		fprintf(file, "history_expression_type=%i\n", history_expression_type);
 	}
 	if(read_default) {
@@ -1218,8 +1289,10 @@ bool QalculateQtSettings::savePreferences(const char *filename, bool is_workspac
 		}
 		if(!default_shortcuts) {
 			for(size_t i = 0; i < keyboard_shortcuts.size(); i++) {
-				if(keyboard_shortcuts[i]->value.empty()) fprintf(file, "keyboard_shortcut=%s %i\n", keyboard_shortcuts[i]->key.toUtf8().data(), keyboard_shortcuts[i]->type);
-				else fprintf(file, "keyboard_shortcut=%s %i %s\n", keyboard_shortcuts[i]->key.toUtf8().data(), keyboard_shortcuts[i]->type, keyboard_shortcuts[i]->value.c_str());
+				for(size_t i2 = 0; i2 < keyboard_shortcuts[i]->type.size(); i2++) {
+					if(keyboard_shortcuts[i]->value[i2].empty()) fprintf(file, "keyboard_shortcut=%s %i\n", keyboard_shortcuts[i]->key.toUtf8().data(), keyboard_shortcuts[i]->type[i2]);
+					else fprintf(file, "keyboard_shortcut=%s %i %s\n", keyboard_shortcuts[i]->key.toUtf8().data(), keyboard_shortcuts[i]->type[i2], keyboard_shortcuts[i]->value[i2].c_str());
+				}
 			}
 		}
 		if(favourite_functions_changed) {
@@ -1248,6 +1321,7 @@ bool QalculateQtSettings::savePreferences(const char *filename, bool is_workspac
 		fprintf(file, "spell_out_logical_operators=%i\n", printops.spell_out_logical_operators);
 		fprintf(file, "caret_as_xor=%i\n", caret_as_xor);
 		fprintf(file, "copy_ascii=%i\n", copy_ascii);
+		fprintf(file, "copy_ascii_without_units=%i\n", copy_ascii_without_units);
 		fprintf(file, "digit_grouping=%i\n", printops.digit_grouping);
 		fprintf(file, "decimal_comma=%i\n", decimal_comma);
 		fprintf(file, "dot_as_separator=%i\n", dot_question_asked ? evalops.parse_options.dot_as_separator : -1);
@@ -1266,6 +1340,7 @@ bool QalculateQtSettings::savePreferences(const char *filename, bool is_workspac
 		fprintf(file, "local_currency_conversion=%i\n", evalops.local_currency_conversion);
 		fprintf(file, "use_binary_prefixes=%i\n", CALCULATOR->usesBinaryPrefixes());
 		fprintf(file, "calculate_as_you_type=%i\n", auto_calculate);
+		if(previous_precision > 0) fprintf(file, "previous_precision=%i\n", previous_precision);
 	}
 	if(read_default) {
 		fprintf(file, "\n[Mode]\n");
@@ -1305,6 +1380,7 @@ bool QalculateQtSettings::savePreferences(const char *filename, bool is_workspac
 		fprintf(file, "warn_about_denominators_assumed_nonzero=%i\n", evalops.warn_about_denominators_assumed_nonzero);
 		fprintf(file, "structuring=%i\n", evalops.structuring);
 		fprintf(file, "angle_unit=%i\n", evalops.parse_options.angle_unit);
+		if(evalops.parse_options.angle_unit == ANGLE_UNIT_CUSTOM && CALCULATOR->customAngleUnit()) fprintf(file, "custom_angle_unit=%s\n", CALCULATOR->customAngleUnit()->referenceName().c_str());
 		fprintf(file, "functions_enabled=%i\n", evalops.parse_options.functions_enabled);
 		fprintf(file, "variables_enabled=%i\n", evalops.parse_options.variables_enabled);
 		fprintf(file, "calculate_functions=%i\n", evalops.calculate_functions);
@@ -1705,6 +1781,15 @@ void QalculateQtSettings::checkVersion(bool force, QWidget *parent) {
 	last_version_check_date.setToCurrentDate();
 }
 
+QString QalculateQtSettings::shortcutText(const std::vector<shortcut_type> &type, const std::vector<std::string> &value) {
+	if(type.size() == 1) return shortcutText(type[0], value[0]);
+	QString str;
+	for(size_t i = 0; i < type.size(); i++) {
+		if(!str.isEmpty()) str += ", ";
+		str += shortcutText(type[i], value[i]);
+	}
+	return str;
+}
 QString QalculateQtSettings::shortcutText(int type, const std::string &value) {
 	if(type < 0) return QString();
 	switch(type) {
@@ -1726,7 +1811,7 @@ QString QalculateQtSettings::shortcutText(int type, const std::string &value) {
 		}
 		default: {}
 	}
-	if(value.empty()) return shortcutTypeText((shortcut_type) type);
+	if(value.empty() || type == SHORTCUT_TYPE_COPY_RESULT) return shortcutTypeText((shortcut_type) type);
 	return tr("%1: %2").arg(shortcutTypeText((shortcut_type) type)).arg(QString::fromStdString(value));
 }
 QString QalculateQtSettings::shortcutTypeText(shortcut_type type) {
@@ -1768,6 +1853,10 @@ QString QalculateQtSettings::shortcutTypeText(shortcut_type type) {
 		case SHORTCUT_TYPE_SCIENTIFIC_NOTATION: {return tr("Activate scientific display mode");}
 		case SHORTCUT_TYPE_ENGINEERING_NOTATION: {return tr("Activate engineering display mode");}
 		case SHORTCUT_TYPE_SIMPLE_NOTATION: {return tr("Activate simple display mode");}
+		case SHORTCUT_TYPE_PRECISION: {return tr("Toggle precision");}
+		case SHORTCUT_TYPE_MAX_DECIMALS: {return tr("Toggle max decimals");}
+		case SHORTCUT_TYPE_MIN_DECIMALS: {return tr("Toggle min decimals");}
+		case SHORTCUT_TYPE_MINMAX_DECIMALS: {return tr("Toggle max/min decimals");}
 		case SHORTCUT_TYPE_RPN_MODE: {return tr("Toggle RPN mode");}
 		case SHORTCUT_TYPE_GENERAL_KEYPAD: {return tr("Show general keypad");}
 		case SHORTCUT_TYPE_PROGRAMMING_KEYPAD: {return tr("Toggle programming keypad");}
@@ -1821,12 +1910,16 @@ QString QalculateQtSettings::shortcutTypeText(shortcut_type type) {
 	}
 	return "-";
 }
-
+void QalculateQtSettings::updateActionValueTexts() {
+	if(copy_action_value_texts.isEmpty()) {
+		copy_action_value_texts << tr("Default") << tr("Formatted result") << tr("Unformatted ASCII result") << tr("Unformatted ASCII result without units") << tr("Formatted expression") << tr("Unformatted ASCII expression") << tr("Formatted expression + result") << tr("Unformatted ASCII expression + result");
+	}
+}
 bool QalculateQtSettings::testShortcutValue(int type, QString &value, QWidget *w) {
+	if(type != SHORTCUT_TYPE_TEXT) value = value.trimmed();
 	switch(type) {
 		case SHORTCUT_TYPE_FUNCTION: {}
 		case SHORTCUT_TYPE_FUNCTION_WITH_DIALOG: {
-			value = value.trimmed();
 			if(value.length() > 2 && value.right(2) == "()") value.chop(2);
 			if(!CALCULATOR->getActiveFunction(value.toStdString())) {
 				QMessageBox::critical(w, QApplication::tr("Error"), QApplication::tr("Function not found."), QMessageBox::Ok);
@@ -1835,7 +1928,6 @@ bool QalculateQtSettings::testShortcutValue(int type, QString &value, QWidget *w
 			break;
 		}
 		case SHORTCUT_TYPE_VARIABLE: {
-			value = value.trimmed();
 			if(!CALCULATOR->getActiveVariable(value.toStdString())) {
 				QMessageBox::critical(w, QApplication::tr("Error"), QApplication::tr("Variable not found."), QMessageBox::Ok);
 				return false;
@@ -1843,7 +1935,6 @@ bool QalculateQtSettings::testShortcutValue(int type, QString &value, QWidget *w
 			break;
 		}
 		case SHORTCUT_TYPE_UNIT: {
-			value = value.trimmed();
 			if(!CALCULATOR->getActiveUnit(value.toStdString())) {
 				QMessageBox::critical(w, QApplication::tr("Error"), QApplication::tr("Unit not found."), QMessageBox::Ok);
 				return false;
@@ -1851,13 +1942,11 @@ bool QalculateQtSettings::testShortcutValue(int type, QString &value, QWidget *w
 			break;
 		}
 		case SHORTCUT_TYPE_OPERATOR: {
-			value = value.trimmed();
 			break;
 		}
 		case SHORTCUT_TYPE_TO_NUMBER_BASE: {}
 		case SHORTCUT_TYPE_INPUT_BASE: {}
 		case SHORTCUT_TYPE_OUTPUT_BASE: {
-			value = value.trimmed();
 			Number nbase; int base;
 			base_from_string(value.toStdString(), base, nbase, type == SHORTCUT_TYPE_INPUT_BASE);
 			if(base == BASE_CUSTOM && nbase.isZero()) {
@@ -1866,6 +1955,37 @@ bool QalculateQtSettings::testShortcutValue(int type, QString &value, QWidget *w
 			}
 			break;
 		}
+		case SHORTCUT_TYPE_PRECISION: {}
+		case SHORTCUT_TYPE_MIN_DECIMALS: {}
+		case SHORTCUT_TYPE_MAX_DECIMALS: {}
+		case SHORTCUT_TYPE_MINMAX_DECIMALS: {
+			bool ok = false;
+			int v = value.toInt(&ok, 10);
+			if(!ok || v < -1 || (type == SHORTCUT_TYPE_PRECISION && v < 2)) {
+				QMessageBox::critical(w, QApplication::tr("Error"), QApplication::tr("Unsupported value."), QMessageBox::Ok);
+				return false;
+			}
+			break;
+		}
+		case SHORTCUT_TYPE_COPY_RESULT: {
+			if(value.isEmpty()) break;
+			int v = -1;
+			if(value.length() > 1) {
+				updateActionValueTexts();
+				v = copy_action_value_texts.indexOf(value);
+				if(v >= 0) value = QString::number(v);
+			} else {
+				bool ok = false;
+				v = value.toInt(&ok, 10);
+				if(!ok) v = -1;
+			}
+			if(v < 0 || v > 7) {
+				QMessageBox::critical(w, QApplication::tr("Error"), QApplication::tr("Unsupported value."), QMessageBox::Ok);
+				value = "";
+				return false;
+			}
+			break;
+		}
 	}
 	return true;
 }
