@@ -310,13 +310,22 @@ void UnitsDialog::newClicked() {
 				else if(!replaced_item->isActive()) emit variableDeactivated((Variable*) replaced_item);
 			} else if(!replaced_item->isActive()) {
 				QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
-				if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
+				if(list.isEmpty()) {
+					list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+					QTreeWidgetItem *i = new QTreeWidgetItem(categoriesView, list.isEmpty() ? NULL : list[0]);
+					i->setText(0, tr("Inactive"));
+					i->setText(1, "Inactive");
+				}
 			}
 		}
 		selected_item = u;
 		if(u->category().empty()) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
-			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+			if(list.isEmpty()) {
+				QStringList l; l << tr("Uncategorized"); l << "Uncategorized";
+				list = categoriesView->findItems("All", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				new QTreeWidgetItem(list.isEmpty() ? categoriesView->topLevelItem(2) : list[0], l);
+			}
 		} else if(u->category() != CALCULATOR->temporaryCategory()) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(u->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
 			if(list.isEmpty()) {
@@ -363,7 +372,12 @@ void UnitsDialog::unitRemoved(Unit *u) {
 }
 void UnitsDialog::unitDeactivated(Unit*) {
 	QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
-	if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
+	if(list.isEmpty()) {
+		list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+		QTreeWidgetItem *i = new QTreeWidgetItem(categoriesView, list.isEmpty() ? NULL : list[0]);
+		i->setText(0, tr("Inactive"));
+		i->setText(1, "Inactive");
+	}
 	unitsModel->invalidate();
 	toModel->invalidate();
 }
@@ -391,13 +405,22 @@ void UnitsDialog::editClicked() {
 				else if(!replaced_item->isActive()) emit variableDeactivated((Variable*) replaced_item);
 			} else if(!replaced_item->isActive()) {
 				QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
-				if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
+				if(list.isEmpty()) {
+					list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+					QTreeWidgetItem *i = new QTreeWidgetItem(categoriesView, list.isEmpty() ? NULL : list[0]);
+					i->setText(0, tr("Inactive"));
+					i->setText(1, "Inactive");
+				}
 			}
 		}
 		selected_item = u;
 		if(u->category().empty()) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
-			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+			if(list.isEmpty()) {
+				QStringList l; l << tr("Uncategorized"); l << "Uncategorized";
+				list = categoriesView->findItems("All", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				new QTreeWidgetItem(list.isEmpty() ? categoriesView->topLevelItem(2) : list[0], l);
+			}
 		} else if(u->category() != CALCULATOR->temporaryCategory()) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(u->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
 			if(list.isEmpty()) {
@@ -503,8 +526,12 @@ void UnitsDialog::deactivateClicked() {
 		if(!list.isEmpty()) {
 			categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
 		} else if(!u->isActive()) {
-				QStringList l; l << tr("Inactive"); l << "Inactive";
-				categoriesView->setCurrentItem(new QTreeWidgetItem(categoriesView, l), 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			QStringList l; l << tr("Inactive"); l << "Inactive";
+			QTreeWidgetItem *i = new QTreeWidgetItem(categoriesView, list.isEmpty() ? NULL : list[0]);
+			i->setText(0, tr("Inactive"));
+			i->setText(1, "Inactive");
+			categoriesView->setCurrentItem(i, 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
 		}
 		emit itemsChanged();
 	}
@@ -791,6 +818,14 @@ void UnitsDialog::updateUnits() {
 	if(selected_category == "User items") {
 		iter->setSelected(true);
 	}
+	if(has_inactive) {
+		//add "Inactive" category if there are inactive units
+		l.clear(); l << tr("Inactive"); l << "Inactive";
+		iter = new QTreeWidgetItem(categoriesView, l);
+		if(selected_category == "Inactive") {
+			iter->setSelected(true);
+		}
+	}
 	l.clear(); l << tr("All", "All units"); l << "All";
 	iter3 = new QTreeWidgetItem(categoriesView, l);
 	tree_struct *item, *item2;
@@ -840,14 +875,6 @@ void UnitsDialog::updateUnits() {
 			iter->setSelected(true);
 		}
 	}
-	if(has_inactive) {
-		//add "Inactive" category if there are inactive units
-		l.clear(); l << tr("Inactive"); l << "Inactive";
-		iter = new QTreeWidgetItem(categoriesView, l);
-		if(selected_category == "Inactive") {
-			iter->setSelected(true);
-		}
-	}
 	iter3->setExpanded(true);
 	if(categoriesView->selectedItems().isEmpty()) {
 		//if no category has been selected (previously selected has been renamed/deleted), select "All"
